/**
 * TypeScript's `Partial<T>` mapped type makes all of the type's properties
 * optional. This variation makes only the specified keys optional.
 *
 * @template T This type will be returned with the properties in U made optional
 * @template U The properties to make optional
 */
export declare type PartialOnly<T, U extends keyof T> = {
    [K in Exclude<keyof T, U>]: T[K];
} & {
    [K in keyof Pick<T, U>]?: T[K];
};
/**
 * TypeScript's `Partial<T>` mapped type makes all of the type's properties
 * optional. This variation makes all properties optional except those
 * specified.
 *
 * @template T This type will be returned with all properties optional except
 *   the properties in U
 * @template U The properties not to make optional
 */
export declare type PartialExcept<T, U extends keyof T> = {
    [K in Exclude<keyof T, U>]?: T[K];
} & {
    [K in keyof Pick<T, U>]: T[K];
};
/**
 * TypeScript's `Pick<T, K>` mapped type returns a type that only includes the
 * specified keys. This type complements it by returning a type with only those
 * keys that are not specified.
 *
 * @template T This type will be returned without the keys specified in U
 * @template U The keys to exclude from T
 */
export declare type Without<T, U extends keyof T> = Pick<T, Exclude<keyof T, U>>;
/**
 * TypeScript's `Pick<T, K>` mapped type returns a type that only includes the
 * specified keys. This type complements it by returning a type with only those
 * keys that are not specified, and makes all of the remaining properties
 * optional as in `Partial<T>`.
 *
 * @template T This type will be returned without the keys specified in U,
 *   with all properties optional
 * @template U The keys to exclude from T
 */
export declare type PartialWithout<T, U extends keyof T> = {
    [K in Exclude<keyof T, U>]?: T[K];
};
/**
 * Update an object's properties with values from an object with a subset of
 * the object's properties. Returns a new object.
 *
 * See the documentation for the Lodash `merge` function for implementation
 * details.
 */
export declare function updateObject<T, U extends keyof T>(original: T, update: Partial<T> | PartialOnly<T, U> | PartialExcept<T, U> | Pick<T, U> | Without<T, U> | PartialWithout<T, U>): T;
